% Ten Tusscher et al model
% Am J Physiol 286: H1573H1589, 2004.
% human ventricular cell 
% improved version of Priebe-Beucklemann
%    
%    t   time variable
%    V   membrane potantial
clear all
load tt_fixed_params
colors = repmat('krgbmc',1,500) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global F R T RTF ;
% Physical constants
F = 96.4867 ;                % Faraday constant, coulombs/mmol
R = 8.314 ;                  % gas constant, J/(K mol)
T = 273+37 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Cm S ;
Cm = 1 ;                      % uF/cm2
S = 0.2 ;                     % surface area to volume ratio

global Vmyo VSR ;
Vmyo = 16.404 ;               % pL
VSR = 1.094 ;                 % pL

global Ko Nao Cao ;
Ko = 5400 ;                   % uM
Nao = 140000 ;                % uM
Cao = 2000 ;                  % uM

global GNa_ PCa_ GK1_ GKr_ GKs_ Gto_ ;
global GpK_ GpCa_ GNab_ GCab_ pKNa ;
GNa_ = 14.838 ;               % nS/pF
PCa_ = 1.75e-4 ;              % cm3 uF-1 s-1 
GK1_ = 5.405 ;                % nS/pF
GKr_ = 0.096 ;                % nS/pF

GpK_ = 0.0146 ;               % nS/pF
GpCa_ = 0.025 ;               % nS/pF
GNab_ = 2.9e-4 ;              % nS/pF
GCab_ = 5.92e-4 ;             % nS/pF

% % % Epicardial cell
Gto_ = 0.294 ;                % nS/pF
GKs_ = 0.245 ;                % nS/pF

% % % % M cell
% Gto_ = 0.294 ;                % nS/pF
% GKs_ = 0.062 ;                % nS/pF
% 
% % % % Endocardial cell
% Gto_ = 0.073 ;                % nS/pF
% GKs_ = 0.245 ;                % nS/pF
% 

% % % % Epicardial cell, M cell
% Gto_ = 0.294 ;                 % nS/pF
% % % % Endocardial cell
% Gto_ = 0.073 ;                 % nS/pF
% 
% % % % Epicardial cell, endocardial cell
% GKs_ = 0.245 ;                 % nS/pF
% % % % M cell
% GKs_ = 0.062 ;                 % nS/pF

pKNa = 0.03 ;                  % relative permeability, Na to K

global INaK_ kNaCa Iup_ Vleak
% Maximum rates of intracellular transport mechanisms
INaK_ = 1.362 ;                % pA/pF
kNaCa = 1000 ;                 % pA/pF
Iup_ = 0.425 ;                 % uM/ms
Vleak = 8e-5 ;                 % ms-1

global KmNa KmCa ksat alpha_ncx eta KmNai KmKo KpCa Kmup ;
% global eta ksat KmNa KmCa KmfCa KmK1 KmNai KmKo KmpCa Kmup  
% global KmNa KmCa KmNai KmKo sigma ;
% % values needed for calculation of NCX current
% Several saturation constants, nomenclature gets tricky
% NCX
KmNa = 87500 ;                 % uM
KmCa = 1380 ;                  % uM
ksat = 0.1 ;                   % unitless
alpha_ncx = 2.5 ;              % unitless
eta = 0.35 ;                   % unitless, actually gamma in paper
% Na-K pump
KmNai = 40000 ;                % uM
KmKo = 1000 ;                  % uM
% Sarcolemmal Ca pump
KpCa = 0.5 ;                    % uM
% SERCA
Kmup = 0.25 ;                  % uM

global arel brel crel ;
arel = 16.464 ;                 % uM/ms
brel = 250 ;                   % uM
crel = 8.232 ;                  % uM/ms

% % % Buffering parameters
% % % Considered generic cytotolic and SR buffers
global Bufc Kbufc BufSR KbufSR 
Bufc = 150 ;                % uM
Kbufc = 1 ;                   % uM
BufSR = 10000 ;              % uM
KbufSR = 300 ;                 % uM 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Set up randomly varying parameter set  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
baselineparameters = [GNa_;GNab_;PCa_;GCab_;Gto_;GKr_;GKs_;GK1_;GpK_; ...
  INaK_;kNaCa;arel;crel;Vleak;Iup_;GpCa_] ;

n_parameters = length(baselineparameters) ;
variations = 300 ;
% variations = 1 ;
sigma = 0.1 ;     % standard deviation of variation of parameters
% sigma = 0 ;

baselinerate = 600 ;
nbaseline = 20 ; % number of stimuli at baseline rate to apply
n_stimuli = nbaseline + 1 ;
%testintervals = [2:2:50,55:5:100] ;
testintervals = [1:1:20] ;
timeafter = 800 ;
stimdur = 2 ;
stim_amp = 25 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 2:  Define simulation and stimulus parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

APDs = zeros(variations,length(testintervals)) ;

for ii=1:variations

  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
  [GNa_,GNab_,PCa_,GCab_,Gto_,GKr_,GKs_,GK1_,GpK_, ...
    INaK_,kNaCa,arel,crel,Vleak,Iup_,GpCa_] = deal(newparamscell{:}) ;

    stim_starts = baselinerate*(0:nbaseline-1)  ;
    stim_ends = stim_starts + stimdur ;

    simints = 2*nbaseline ;
    for i=1:nbaseline
      intervals(2*i-1,:) = [stim_starts(i),stim_ends(i)] ;
      intervals(2*i,:) = [stim_ends(i),baselinerate*i] ;
    end 
    
    Istim = zeros(simints,1) ;
    stimindices = 2*(1:nbaseline) - 1 ;
    Istim(stimindices) = -stim_amp ;

  % Initial conditions
  %% Don't matter that much since cell will rest for 60 s before stimulus
V = -85 ;
m = 0 ;
h = 1 ;
j = 1 ;
d = 0 ;
f = 1 ;
fCa = 1 ;
r = 0 ;
s = 1 ;
xs = 0 ;
xr1 = 1 ;
xr2 = 1 ;
g = 0 ;
Cai = 0.1 ;
CaSR = 1000 ;
Nai = 10000 ;
Ki = 130000 ;

statevar_i = [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] ;

[post,posstatevars] = ode15s(@dydt_tentusscher,[0,60000],statevar_i,[],0) ;
  
  statevar_i = posstatevars(end,:) ;

  t = 0 ;
  statevars = statevar_i ;
  for i=1:simints-2
    [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;  
    
    t = post(end) ;
    statevar_i = posstatevars(end,:) ;
  end
  statevar_keep = statevar_i ;
  statevars = statevar_i ;
  for i=simints-1:simints
    [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;  
    t = [t;post(2:end)] ;
    statevars = [statevars;posstatevars(2:end,:)] ;
    statevar_i = posstatevars(end,:) ;
  end      

outputcell = num2cell(statevars,1) ;
[V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;
clear statevars

tregular = min(t):0.1:max(t) ;
Vregular = interp1(t,V,tregular) ;

above72dex = find(Vregular > -72) ;
duration(ii) = tregular(above72dex(end)) - tregular(above72dex(1)) ;

% actionpotentials{2*ii-1} = t ;
% actionpotentials{2*ii} = V ;

  for iii = 1:length(testintervals)
    intervals(1,:) = [0,stimdur] ;
    intervals(2,:) = [stimdur,duration(ii) + testintervals(iii)] ;
    intervals(3,:) = [duration(ii) + testintervals(iii), ...
        duration(ii) + testintervals(iii) + stimdur] ;
    intervals(4,:) = [duration(ii) + testintervals(iii) + stimdur, ...
        duration(ii) + testintervals(iii) + stimdur + timeafter] ;
    Istim = [-stim_amp,0,-stim_amp,0] ;
    t = 0 ;
    statevar_i = statevar_keep ;
    statevars = statevar_i ;
   for i=1:4
      [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
      t = [t;post(2:end)] ;
      statevars = [statevars;posstatevars(2:end,:)] ;
      statevar_i = posstatevars(end,:) ;
    end      
    outputcell = num2cell(statevars,1) ;
    [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;

    tregular = min(t):0.1:max(t) ;
    Vregular = interp1(t,V,tregular) ;

    above72dex = find(Vregular > -72) ;
    APDs(ii,iii) = tregular(above72dex(end))-testintervals(iii) - duration(ii) ;

    % figure(ii)
     %hold on
    % plot(t,V,colors(mod(iii,length(colors))+1))
  end
  APDderiv = diff(APDs(ii,:))./diff(testintervals) ;
  maxslopes(ii) = max(APDderiv) ;
  slopegreater = find(APDderiv > 1) ;
  if(slopegreater)
    vulnerability(ii) = testintervals(slopegreater(end)) ;
  else
    vulnerability(ii) = 0 ;
  end

if (mod(ii,50) == 0)
  disp(['Completed ',int2str(ii),' iterations'])
end

end

%  figure
%  plot(testintervals,APDs,'o-')

save restitution_fixed allparameters maxslopes vulnerability APDs variations
